@@ -21,7 +21,7 @@ try:
 	from pyanthem.pyanthem_vars import *
 except:
 	from pyanthem_vars import *
-from google_drive_downloader import GoogleDriveDownloader as gdd
+import gdown
 import subprocess as sp
 import PIL.Image as Image
 
@@ -94,6 +94,7 @@ def run(display=True):
 	Main command to run GUI or CLI
 	'''
 	root=GUI(display=display)
+	##root.configure(bg="white")
 	sys.ps1='♫ '
 	if display:
 		root.mainloop()
@@ -113,11 +114,11 @@ class GUI(ThemedTk):
 		if not os.path.isdir(self.sf_path):
 			print('Initializing soundfont library...')
 			os.mkdir(self.sf_path)
-			gdd.download_file_from_google_drive(file_id=sound_font,dest_path=\
-				os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)),'anthem_soundfonts'),'font.sf2'),showsize=True)
+			gdown.download('https://drive.google.com/uc?id={}'.format(sound_font),
+				os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)),'anthem_soundfonts'),'font.sf2'),quiet=False)
 		if self.display:
 			ThemedTk.__init__(self)
-			self.set_theme('clearlooks')
+			self.set_theme('breeze')
 			self.initGUI()
 	
 	def quit(self,event=None):
@@ -159,7 +160,7 @@ class GUI(ThemedTk):
 		all StringVars and IntVars to a new dict called 'self.cfg', that can be accessed 
 		oustide the GUI and dumped to a pickle file, which essentially "saves" the GUI.
 		'''
-		self.cfg={k: getattr(self,k).get() if self_fns[k] is 'entry' else getattr(self,k) for k in self_fns}
+		self.cfg={k: getattr(self,k).get() if self_fns[k] == 'entry' else getattr(self,k) for k in self_fns}
 
 	def dump_cfg(self):
 		'''
@@ -169,19 +170,19 @@ class GUI(ThemedTk):
 		pickle.dump(self.cfg,open(file_out, "wb"))
 		self.message(f'cfg file saved to {file_out}')
 	
-	def load_data(self,filein=None):
+	def load_data(self,file_in=None):
 		'''
-		Loads dataset from filein. At the time, only supports .mat files.
+		Loads dataset from file_in. At the time, only supports .mat files.
 		'''
-		if filein is None:
-			filein=uiopen(title='Select mat or hdf5 file for import',filetypes=[('.mat files','*.mat'),('hdf5 files','*.h5'),('hdf5 files','*.hdf5')])
-		if filein=='.':
+		if file_in is None:
+			file_in=uiopen(title='Select mat or hdf5 file for import',filetypes=[('.mat files','*.mat'),('hdf5 files','*.h5'),('hdf5 files','*.hdf5')])
+		if file_in=='.':
 			return
-		if filein.endswith('.mat'):
-			data_unk,var=loadmat(filein),whosmat(filein)
+		if file_in.endswith('.mat'):
+			data_unk,var=loadmat(file_in),whosmat(file_in)
 			var = [v[0] for v in var]
-		elif filein.endswith('.hdf5') or filein.endswith('.h5'):
-			data_unk=h5py.File(filein, 'r')
+		elif file_in.endswith('.hdf5') or file_in.endswith('.h5'):
+			data_unk=h5py.File(file_in, 'r')
 			var = data_unk.keys
 		data = {}
 		for k in var:
@@ -220,22 +221,23 @@ class GUI(ThemedTk):
 		'''
 		GUI-addons for load_data. Prompts user with filedialog, assigns defaults and sets GUI fields. 
 		'''
-		filein=uiopen(title='Select mat or hdf5 file for import',filetypes=[('.mat files','*.mat'),('hdf5 files','*.h5'),('hdf5 files','*.hdf5')])
-		if filein=='.':
+		file_in=uiopen(title='Select mat or hdf5 file for import',filetypes=[('.mat files','*.mat'),('hdf5 files','*.h5'),('hdf5 files','*.hdf5')])
+		if file_in=='.':
 			return
-		self.load_data(filein)
+		self.load_data(file_in)
 		if not hasattr(self,'data'):
 			return
+
 		self.data['H_pp']=self.data['H']
 		self.data['H_fp']=self.data['H']
 		self.fr.set(self.data['fr'])
 		if 'W' in self.data:
 			self.data['W_pp']=self.data['W']
-		self.file_in.set(os.path.splitext(os.path.split(filein)[1])[0])
+		self.file_in.set(os.path.splitext(os.path.split(file_in)[1])[0])
 
 		# Set some defaults
 		self.file_out.set(self.file_in.get())
-		self.save_path.set(os.path.split(filein)[0])
+		self.save_path.set(os.path.split(file_in)[0])
 		Hstr='H' # for whatever reason, can't double nest quotations in an f-string :/
 		self.brightness.set(f'{float(f"{np.mean(self.data[Hstr])+np.std(self.data[Hstr]):.3g}"):g}')
 		self.threshold.set(f'{float(f"{np.mean(self.data[Hstr])+np.std(self.data[Hstr]):.3g}"):g}')
@@ -243,23 +245,23 @@ class GUI(ThemedTk):
 		self.init_plots()
 		self.process_H_W()
 	
-	def load_config(self,filein=None):
+	def load_config(self,file_in=None):
 		'''
 		Loads .p file containing dict of parameters needed to create outputs. If display=True, sets GUI fields.
 		'''
-		if filein is None:
-			filein=uiopen(title='Select pickle file for import',filetypes=[('pickle file','*.p'),('pickle file','*.pkl'),('pickle file','*.pickle')])
-		if filein=='.':
+		if file_in is None:
+			file_in=uiopen(title='Select pickle file for import',filetypes=[('pickle file','*.p'),('pickle file','*.pkl'),('pickle file','*.pickle')])
+		if file_in=='.':
 			return
-		with open(filein, "rb") as f:
+		with open(file_in, "rb") as f:
 			self.cfg=pickle.load(f)
 			if self.display:
 				for key,value in self.cfg.items():
-					if self_fns[key] is 'entry':
+					if self_fns[key] == 'entry':
 						getattr(self,key).set(value)
 					else:
 						setattr(self,key,value)
-				self.refresh_GUI()
+				self.process_H_W()
 			else:
 				return self
 
@@ -396,6 +398,7 @@ class GUI(ThemedTk):
 		# Colormap
 		cmap=getattr(cmaps,self.cfg['cmapchoice'])
 		self.cmap=cmap(np.linspace(0,1,len(self.data['H_pp'])))
+		self.message('Updated.')
 		if self.display:
 			self.refresh_GUI()
 
@@ -430,7 +433,7 @@ class GUI(ThemedTk):
 			track.append(mido.Message('note_off', note=self.keys[i], time=250))
 		track.append(mido.Message('note_off', note=self.keys[i], time=500))
 		mid.save(fn_midi)
-		cmd='fluidsynth -ni {} -F {} -r {} {} {} '.format(self.cfg['fluidsynthextracommand'],fn_wav,fs,fn_font,fn_midi)
+		cmd='fluidsynth -ni {} -F "{}" -r {} "{}" "{}"'.format(self.cfg['fluidsynthextracommand'],fn_wav,fs,fn_font,fn_midi)
 		os.system(cmd)
 		music.load(fn_wav)
 		for i in range(len(self.keys)):
@@ -502,7 +505,7 @@ class GUI(ThemedTk):
 			for chan in range(nchan):
 				track.append(mido.Message('note_off', channel=chan, note=self.keys[chan], time=0))
 			mid.save(fn_midi)
-		os.system('fluidsynth -ni {} -F {} -r {} {} {}'.format(self.cfg['fluidsynthextracommand'],fn_wav,fs,fn_font,fn_midi))
+		os.system('fluidsynth -ni {} -F "{}" -r {} "{}" "{}"'.format(self.cfg['fluidsynthextracommand'],fn_wav,fs,fn_font,fn_midi))
 		self.message(f'Audio file written to {self.cfg["save_path"]}')
 		return self
 
@@ -518,7 +521,7 @@ class GUI(ThemedTk):
 		fn_vid=os.path.join(self.cfg['save_path'],self.cfg['file_out'])+'.mp4'
 		v_shape=self.data['W_shape'][::-1][1:] # Reverse because ffmpeg does hxw
 		command=['ffmpeg',
-			'-loglevel', 'warning', # Prevents excessive messages
+			'-loglevel', 'fatal', # Prevents excessive messages
 			'-hide_banner',
 			'-y', # Auto overwrite
 			'-f', 'image2pipe',
@@ -553,18 +556,19 @@ class GUI(ThemedTk):
 		Merges video and audio with ffmpeg
 		'''
 		fn=os.path.join(self.cfg['save_path'],self.cfg['file_out'])
-		cmd='ffmpeg -hide_banner -loglevel warning -y -i {} -i {} -c:a aac -map 0:v:0 -map 1:a:0 {}'.format(fn+'.mp4',fn+'.wav',fn+'_AV.mp4')
+		cmd='ffmpeg -hide_banner -loglevel fatal -y -i "{}" -i "{}" -shortest -c:a aac -map 0:v:0 -map 1:a:0 "{}"'.format(fn+'.mp4',fn+'.wav',fn+'_AV.mp4')
 		os.system(cmd)
 		self.message(f'A/V file written to {self.cfg["save_path"]}')
 		return self
 	
 	def write_AV(self):
 		'''
-		Runs full write and merge
+		Runs full write and merge with cleanup
 		'''
 		self.write_video()
 		self.write_audio()
 		self.merge()
+		self.cleanup()
 		if not self.display:
 			return self
 
@@ -614,7 +618,7 @@ class GUI(ThemedTk):
 		self.octave_add=init_entry('2')
 		self.scale_type=init_entry('Maj. 7 (4/oct)')
 		self.key=init_entry('C')
-		self.sound_preset=init_entry('Piano')
+		self.sound_preset=init_entry('Yamaha C5 Grand')
 		self.comps_to_show=init_entry('all')
 		self.cmapchoice=init_entry('jet')
 		
@@ -652,10 +656,14 @@ class GUI(ThemedTk):
 		Entry(textvariable=self.file_out).grid(row=5, column=1,columnspan=2,sticky='W')
 		Entry(textvariable=self.save_path,width=17).grid(row=7, column=1,columnspan=2,sticky='EW')
 		Entry(textvariable=self.speed,width=7).grid(row=2, column=4, sticky='W')
-		Entry(textvariable=self.start_percent,width=7).grid(row=3, column=4, sticky='W')
-		Entry(textvariable=self.end_percent,width=7).grid(row=4, column=4, sticky='W')
-		Entry(textvariable=self.baseline,width=7).grid(row=5, column=4, sticky='W')
-		Entry(textvariable=self.brightness,width=7).grid(row=6, column=4, sticky='W')
+		self.start_percent_entry=Entry(textvariable=self.start_percent,width=7)
+		self.start_percent_entry.grid(row=3, column=4, sticky='W')
+		self.end_percent_entry=Entry(textvariable=self.end_percent,width=7)
+		self.end_percent_entry.grid(row=4, column=4, sticky='W')
+		self.baseline_entry=Entry(textvariable=self.baseline,width=7)
+		self.baseline_entry.grid(row=5, column=4, sticky='W')
+		self.brightness_entry=Entry(textvariable=self.brightness,width=7)
+		self.brightness_entry.grid(row=6, column=4, sticky='W')
 		self.threshold_entry=Entry(textvariable=self.threshold,width=7)
 		self.threshold_entry.grid(row=2, column=6, sticky='W')
 
@@ -696,7 +704,7 @@ class GUI(ThemedTk):
 		self.key_menu.grid(row=5, column=6, sticky='W')
 		self.key_menu.current()
 
-		self.sound_preset_menu=Combobox(self,textvariable=self.sound_preset,width=7)
+		self.sound_preset_menu=Combobox(self,textvariable=self.sound_preset,width=11)
 		self.sound_preset_menu['values']=tuple(sound_presets.keys())
 		self.sound_preset_menu['state']='readonly'
 		self.sound_preset_menu.grid(row=6, column=6, sticky='W')
@@ -712,15 +720,14 @@ class GUI(ThemedTk):
 		filemenu=Menu(menubar, tearoff=0)
 		filemenu.add_command(label="Load data...", command=self.load_GUI)
 		filemenu.add_command(label="Load config...", command=self.load_config)
+		filemenu.add_command(label="Load raw...", command=self.process_raw)
 		filemenu.add_command(label="Quit",command=self.quit,accelerator="Ctrl+Q")
 
 		savemenu=Menu(menubar, tearoff=0)
 		savemenu.add_command(label="Audio", command=self.write_audio)
 		savemenu.add_command(label="Video", command=self.write_video)
-		savemenu.add_command(label="Config", command=self.dump_cfg)
-		savemenu.add_command(label="Merge A/V", command=self.merge)
-		savemenu.add_command(label="Write A/V then merge", command=self.write_AV)
-		savemenu.add_command(label="Cleanup", command=self.cleanup)
+		savemenu.add_command(label="Audio + Video", command=self.write_AV)
+		savemenu.add_command(label="Config File", command=self.dump_cfg)
 
 		advancedmenu=Menu(menubar, tearoff=0)
 		advancedmenu.add_command(label="Custom fluidsynth params", command=self.fluidsynthextra)
@@ -748,6 +755,22 @@ class GUI(ThemedTk):
 		self.bind_all("<Control-q>", self.quit)
 		self.bind_all("<Control-a>", lambda:[self.process_H_W(),self.refresh_GUI()])
 
+		# Callbacks
+		def callback(event):
+   			self.process_H_W()
+		self.cmapchooser.bind("<<ComboboxSelected>>", callback)
+		self.brightness_entry.bind("<FocusOut>",callback)
+		self.brightness_entry.bind("<Return>",callback)
+		self.threshold_entry.bind("<FocusOut>",callback)
+		self.threshold_entry.bind("<Return>",callback)
+		self.start_percent_entry.bind("<FocusOut>",callback)
+		self.start_percent_entry.bind("<Return>",callback)
+		self.end_percent_entry.bind("<FocusOut>",callback)
+		self.end_percent_entry.bind("<Return>",callback)
+		self.baseline_entry.bind("<FocusOut>",callback)
+		self.baseline_entry.bind("<Return>",callback)
+
+
 	def init_plots(self):
 		'''
 		Initializes the plot areas. Is called every time update_GUI() is called.
@@ -815,12 +838,24 @@ class GUI(ThemedTk):
 		self.comps_to_show_entry=Entry(textvariable=self.comps_to_show,width=15,justify='center')
 		self.comps_to_show_entry.grid(row=30, column=6, columnspan=1,sticky='W')
 		
-	def process_raw(self,data=None,n_clusters=None,frame_rate=None,save=False,file_in=None):
+	def process_raw(self,file_in=None,n_clusters=None,save=False):
 		'''
 		Decomposes raw dataset. Can be used in two ways: as a part of the 
 		GUI class for immediate processing (e.g. process_raw().write_AV()),
 		or as a method to save a new dataset. 
 		'''
+		if file_in is None:
+			file_in=uiopen(title='Select mat or hdf5 file for import',filetypes=[('.mat files','*.mat'),('hdf5 files','*.h5'),('hdf5 files','*.hdf5')])
+
+		if file_in.endswith('.mat'):
+			data_unk,var=loadmat(file_in),whosmat(file_in)
+			var = [v[0] for v in var]
+		elif file_in.endswith('.hdf5') or file_in.endswith('.h5'):
+			data_unk=h5py.File(file_in, 'r')
+			var = data_unk.keys
+		if len(var) != 1:
+			self.message('ERROR: Please use a file with only 1 variable that is 3D')
+		data=np.asarray(data_unk[var[0]])
 		sh=data.shape
 		if len(sh) != 3:
 			self.message('ERROR: input dataset is not 3D.')
@@ -831,10 +866,11 @@ class GUI(ThemedTk):
 		data_nn=data[~nanidx] # nn=non-nan
 		# k-means
 		print('Performing k-means...',end='')
-		if n_clusters is None:
-			# Default k is the 4th root of the number of samples per frame (for 256x256, this would be 16)
-			n_clusters=int(len(data)**.25) 
-			print(f'No num_clusters given. Defaulting to {n_clusters}...',end='')
+		n_clusters=int(sd.askstring('Imput # of clusters for decomposition...','',initialvalue=int(len(data)**.25),parent=self))
+		# if n_clusters is None:
+		# 	# Default k is the 4th root of the number of samples per frame (for 256x256, this would be 16)
+		# 	n_clusters=int(len(data)**.25) 
+		# 	print(f'No num_clusters given. Defaulting to {n_clusters}...',end='')
 		idx_nn=KMeans(n_clusters=n_clusters, random_state=0).fit(data_nn).labels_
 		idx=np.zeros((len(data),))
 		idx[nanidx==False]=idx_nn
@@ -861,25 +897,48 @@ class GUI(ThemedTk):
 		print('done.')
 		
 		# Assign variables and save
-		self.data={}
-		self.data['H']=H
-		self.data['W']=W.reshape(sh[0],sh[1],n_clusters)
-		#self.data['W_shape']=self.data['W'].shape.flatten()
-		if frame_rate==[]:
-			self.data['fr']=10
-			print('No fr given. Defaulting to 10')
+		data={}
+		data['H']=H
+		data['W']=W.reshape(sh[0],sh[1],n_clusters)
+		# Checks inner dimension match if both H and W present in file.
+		# try flipping dims
+		if data['H'].T.shape[0] == data['W'].T.shape[-1]:
+			data['H'] = data['H'].T
+			data['W'] = data['W'].T
 		else:
-			self.data['fr']=frame_rate
+			self.message('Error: Inner or outer dimensions of W [shape={}] and H [shape={}] do not match!'.format(data['H'].shape, data['W'].shape))
+			
+		data['W_shape']=data['W'].shape
+		data['W']=data['W'].reshape(data['W'].shape[0]*data['W'].shape[1],data['W'].shape[2])
+		data['fr']=data['H'].shape[1]/60 # Forces length to be 1 minute!
+		self.data=data
+
+		if not self.display:
+			return self
+
 		if save:
-			if file_in is None:
-				file_in='data.mat'
 			fn=file_in.replace('.mat','_decomp.mat')
 			savemat(fn,self.data)
 			self.message(f'Decomposed data file saved to {fn}')
 		
-		# Reshape W here, since any use of self from here would require a flattened W
-		self.data['W']=self.data['W'].reshape(self.data['W'].shape[0]*self.data['W'].shape[1],self.data['W'].shape[2])
-		return self
+		if self.display:
+			self.data['H_pp']=self.data['H']
+			self.data['H_fp']=self.data['H']
+			self.fr.set(self.data['fr'])
+			self.data['W_pp']=self.data['W']
+			self.file_in.set(os.path.splitext(os.path.split(file_in)[1])[0])
+
+			# Set some defaults
+			self.file_out.set(self.file_in.get())
+			self.save_path.set(os.path.split(file_in)[0])
+			Hstr='H' # for whatever reason, can't double nest quotations in an f-string :/
+			self.brightness.set(f'{float(f"{np.mean(self.data[Hstr])+np.std(self.data[Hstr]):.3g}"):g}')
+			self.threshold.set(f'{float(f"{np.mean(self.data[Hstr])+np.std(self.data[Hstr]):.3g}"):g}')
+			self.comps_to_show_arr=list(range(len(self.data['H'])))
+			self.init_plots()
+			self.process_H_W()
+		else:
+			return self
 
 	def fluidsynthextra(self):
 		self.fluidsynthextracommand=sd.askstring('Input custom fluidsynth parameters here','',initialvalue=self.fluidsynthextracommand,parent=self)
@@ -904,4 +963,4 @@ if __name__=="__main__":
 	run()
 
 # self\.([a-z_]{1,14})\.get\(\)
-# self\.cfg\[$1\]
\ No newline at end of file
+# self\.cfg\[$1\]
